import torch

tensor_repr = torch.Tensor.__repr__


def tensor_info(tensor: torch.Tensor) -> str:
    """Return human-friendly debug info of the torch.tensor.
    :param tensor torch.Tensor: The tensor to look into.
    :return str: The string representation of the tensor.

    >>> tensor = torch.arange(3*4).reshape(3, 4)
    >>> print(tensor_info(tensor))
    <Tensor: shape=(3, 4), dtype=int64, device='cpu'>
    tensor([[ 0,  1,  2,  3],
            [ 4,  5,  6,  7],
            [ 8,  9, 10, 11]])
    """
    attributes = [
        f"shape={tuple(tensor.shape)}",
        f"dtype={repr(tensor.dtype)[6:]}",
        f"device='{tensor.device}'",
    ]
    if tensor.requires_grad:
        attributes.append("requires_grad=True")

    return f"""\
<{tensor.__class__.__name__}: {', '.join(attributes)}>
{tensor.__repr__()}"""


# torch.Tensor.__repr__ = tensor_info



import numpy as np

ndarray_repr = np.ndarray.__repr__


def ndarray_info(array: np.ndarray) -> str:
    """Return human-friendly debug info of the ndarray.
    :param array np.ndarray: The array to look into.
    :return str: The string representation of the array.

    >>> arr = np.arange(3*4).reshape(3, 4)
    >>> print(ndarray_info(arr))
    <ndarray: shape=(3, 4), dtype=int64>
    array([[ 0,  1,  2,  3],
           [ 4,  5,  6,  7],
           [ 8,  9, 10, 11]])

    >>> arr = np.arange(10000).reshape(100,100)
    >>> print(ndarray_info(arr))
    <ndarray: shape=(100, 100), dtype=int64>
    array([[   0,    1,    2, ...,   97,   98,   99],
           [ 100,  101,  102, ...,  197,  198,  199],
           [ 200,  201,  202, ...,  297,  298,  299],
           ...,
           [9700, 9701, 9702, ..., 9797, 9798, 9799],
           [9800, 9801, 9802, ..., 9897, 9898, 9899],
           [9900, 9901, 9902, ..., 9997, 9998, 9999]])
    """
    attributes = [
        f"shape={array.shape}",
        f"dtype={array.dtype}",
    ]
    return f"""\
<{array.__class__.__name__}: {', '.join(attributes)}>
{array.__repr__()}"""

    r"""
    data = ndarray_repr(array)
    data = re.findall(r'\[.*\]', data, flags=re.MULTILINE | re.DOTALL)[0]
    attributes.append(f"data=\n      {data}")

    return f'''array({', '.join(attributes)})'''
    """



#!/usr/bin/env python

"""Tests for `arraydebug` package."""

import numpy as np
import unittest
import arraydebug


class TestArraydebug(unittest.TestCase):
    def setUp(self) -> None:
        arraydebug.register_reprs()
        arraydebug.inject_repr()

    def tearDown(self) -> None:
        arraydebug.inject_repr()

    def test(self):
        self.assertEqual(repr(1), "1")
        self.assertEqual(repr("abc"), "'abc'")
        self.assertEqual(repr([1, 2, 3, 4]), "[1, 2, 3, 4]")

    def test_recover_repr(self):
        arr = np.arange(4)

        arraydebug.inject_repr()
        self.assertEqual(repr(arr), "<ndarray: shape=(4,), dtype=int64>\narray([0, 1, 2, 3])")

        arraydebug.recover_repr()
        self.assertEqual(repr(arr), "array([0, 1, 2, 3])")



import sys as sys
import builtins as _builtins
from typing import Type, TypeVar, Callable
from arraydebug.array_like import array_like_info

T = TypeVar("T")


def register_repr(cls: Type[T], info_fn: Callable[[T], str]) -> None:
    repr_fn_table[cls] = info_fn


def register_reprs() -> None:
    # if torch is already loaded
    if "torch" in sys.modules:
        from arraydebug.torch import tensor_info

        torch = sys.modules["torch"]
        register_repr(torch.Tensor, tensor_info)

    # if numpy is already loaded
    if "numpy" in sys.modules:
        from arraydebug.numpy import ndarray_info

        np = sys.modules["numpy"]
        register_repr(np.ndarray, ndarray_info)


_default_repr = _builtins.repr


def _new_repr(obj) -> str:
    if type(obj) in repr_fn_table:
        return repr_fn_table[type(obj)](obj)
    elif not isinstance(obj, type) and "__array__" in dir(obj):
        return array_like_info(obj)
    else:
        return _default_repr(obj)


def recover_repr() -> None:
    _builtins.repr = _default_repr


def inject_repr() -> None:
    _builtins.repr = _new_repr


repr_fn_table = {}



import pandas as pd
import numpy as np
import unittest
from inspect import cleandoc

import arraydebug  # noqa


class TestArrayLike(unittest.TestCase):
    def test_pd_dataframe(self):
        """
        <DataFrame: shape=(6, 4), dtype=int64>
                       A     B     C     D
        2013-01-01  1000  1001  1002  1003
        2013-01-02  1004  1005  1006  1007
        2013-01-03  1008  1009  1010  1011
        2013-01-04  1012  1013  1014  1015
        2013-01-05  1016  1017  1018  1019
        2013-01-06  1020  1021  1022  1023
        """
        df = pd.DataFrame(
            np.arange(24).reshape(6, 4) + 1000,
            index=pd.date_range("20130101", periods=6),
            columns=list("ABCD"),
        )
        self.assertEqual(repr(df), cleandoc(self.test_pd_dataframe.__doc__))



import numpy as np
import torch
import pandas as pd

# import arraydebug after numpy, torch, and pandas
# you may add '# noqa' comment to suppress "imported but unused" warning
import arraydebug  # noqa


def fn():
    arr = np.arange(24).reshape(6, 4) + 1000

    tensor = torch.rand(6, 4, requires_grad=True)

    df = pd.DataFrame(arr, index=pd.date_range("20130101", periods=6), columns=list("ABCD"))

    print(repr(arr), "\n")
    print(repr(tensor), "\n")
    print(repr(df), "\n")


fn()



import torch
import unittest
from inspect import cleandoc

import arraydebug  # noqa


class TestTorch(unittest.TestCase):
    def setUp(self) -> None:
        arraydebug.register_reprs()
        arraydebug.inject_repr()

    def test_1D(self):
        """
        <Tensor: shape=(4,), dtype=int64, device='cpu'>
        tensor([0, 1, 2, 3])
        """
        arr = torch.arange(4)
        self.assertEqual(repr(arr), cleandoc(self.test_1D.__doc__))

    def test_1D_long(self):
        """
        <Tensor: shape=(10000,), dtype=float32, device='cpu'>
        tensor([0.0000e+00, 1.0000e+00, 2.0000e+00,  ..., 9.9970e+03, 9.9980e+03,
                9.9990e+03])
        """
        arr = torch.arange(10000, dtype=torch.float32)
        self.assertEqual(repr(arr), cleandoc(self.test_1D_long.__doc__))

    def test_2D(self):
        """
        <Tensor: shape=(3, 4), dtype=int64, device='cpu'>
        tensor([[ 0,  1,  2,  3],
                [ 4,  5,  6,  7],
                [ 8,  9, 10, 11]])
        """
        arr = torch.arange(3 * 4).reshape(3, 4)
        self.assertEqual(repr(arr), cleandoc(self.test_2D.__doc__))

    def test_2D_long(self):
        """
        <Tensor: shape=(100, 100), dtype=int64, device='cpu'>
        tensor([[   0,    1,    2,  ...,   97,   98,   99],
                [ 100,  101,  102,  ...,  197,  198,  199],
                [ 200,  201,  202,  ...,  297,  298,  299],
                ...,
                [9700, 9701, 9702,  ..., 9797, 9798, 9799],
                [9800, 9801, 9802,  ..., 9897, 9898, 9899],
                [9900, 9901, 9902,  ..., 9997, 9998, 9999]])
        """
        arr = torch.arange(10000).reshape(100, 100)
        self.assertEqual(repr(arr), cleandoc(self.test_2D_long.__doc__))

    def test_requires_grad(self):
        """
        <Tensor: shape=(3, 4), dtype=float64, device='cpu', requires_grad=True>
        tensor([[ 0.,  1.,  2.,  3.],
                [ 4.,  5.,  6.,  7.],
                [ 8.,  9., 10., 11.]], dtype=torch.float64, requires_grad=True)
        """
        arr = torch.arange(3 * 4, dtype=torch.float64).reshape(3, 4)
        arr.requires_grad_(True)
        self.assertEqual(repr(arr), cleandoc(self.test_requires_grad.__doc__))



from arraydebug.core import (  # noqa
    recover_repr,
    inject_repr,
    register_reprs,
    register_repr,
    repr_fn_table,
)

__author__ = """Qimai Li"""
__email__ = "liqimai@qq.com"
__version__ = "0.1.1"


register_reprs()
inject_repr()



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("HISTORY.md") as history_file:
    history = history_file.read()

with open("requirements.txt") as requirements_file:
    requirements = requirements_file.read().splitlines()

with open("requirements_dev.txt") as requirements_dev_file:
    test_requirements = requirements_dev_file.read().splitlines()

setup(
    author="Qimai Li",
    author_email="liqimai@qq.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    description="ArrayDebug generates human-friendly debug information for array-like objects.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords="arraydebug",
    name="arraydebug",
    package_dir={"": "src"},
    packages=find_packages(where="src", include=["arraydebug", "arraydebug.*"]),
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/liqimai/arraydebug",
    project_urls={
        "Bug Tracker": "https://github.com/liqimai/arraydebug/issues",
    },
    version="0.1.1",
    zip_safe=True,
)



import numpy as np
import unittest
from inspect import cleandoc

import arraydebug  # noqa


class TestNumpy(unittest.TestCase):
    def setUp(self) -> None:
        arraydebug.register_reprs()
        arraydebug.inject_repr()

    def test_1D(self):
        """
        <ndarray: shape=(4,), dtype=int64>
        array([0, 1, 2, 3])
        """
        arr = np.arange(4)
        self.assertEqual(repr(arr), cleandoc(self.test_1D.__doc__))

    def test_1D_long(self):
        """
        <ndarray: shape=(10000,), dtype=float32>
        array([0.000e+00, 1.000e+00, 2.000e+00, ..., 9.997e+03, 9.998e+03,
               9.999e+03], dtype=float32)
        """
        arr = np.arange(10000, dtype=np.float32)
        self.assertEqual(repr(arr), cleandoc(self.test_1D_long.__doc__))

    def test_2D(self):
        """
        <ndarray: shape=(3, 4), dtype=int64>
        array([[ 0,  1,  2,  3],
               [ 4,  5,  6,  7],
               [ 8,  9, 10, 11]])
        """
        arr = np.arange(3 * 4).reshape(3, 4)
        self.assertEqual(repr(arr), cleandoc(self.test_2D.__doc__))

    def test_2D_long(self):
        """
        <ndarray: shape=(100, 100), dtype=int64>
        array([[   0,    1,    2, ...,   97,   98,   99],
               [ 100,  101,  102, ...,  197,  198,  199],
               [ 200,  201,  202, ...,  297,  298,  299],
               ...,
               [9700, 9701, 9702, ..., 9797, 9798, 9799],
               [9800, 9801, 9802, ..., 9897, 9898, 9899],
               [9900, 9901, 9902, ..., 9997, 9998, 9999]])
        """
        arr = np.arange(10000).reshape(100, 100)
        self.assertEqual(repr(arr), cleandoc(self.test_2D_long.__doc__))



def array_like_info(array_like) -> str:
    """
    Return human-friendly debug info of the array_like object.
    :param array_like:
        The array_like object to look into, such as pandas.DataFrame.
    :return str: the string representation of the array_like object.

    >>> import pandas as pd
    >>> import numpy as np
    >>> df = pd.DataFrame(
    ...     np.arange(24).reshape(6, 4) + 1000,
    ...     index=pd.date_range("20130101", periods=6),
    ...     columns=list("ABCD"))
    >>> print(array_like_info(df))
    <DataFrame: shape=(6, 4), dtype=int64>
                   A     B     C     D
    2013-01-01  1000  1001  1002  1003
    2013-01-02  1004  1005  1006  1007
    2013-01-03  1008  1009  1010  1011
    2013-01-04  1012  1013  1014  1015
    2013-01-05  1016  1017  1018  1019
    2013-01-06  1020  1021  1022  1023
    """
    array = array_like.__array__()
    return f"""\
<{array_like.__class__.__name__}: shape={array.shape}, dtype={array.dtype}>
{array_like.__repr__()}"""
